package fwadmin;

/*

  JFwadmin v 0.8

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.util.StringTokenizer;
import java.util.Vector;

public class InputOutputRule extends Object implements Comparable
{
  protected static Vector miscRulesVector = new Vector();
  protected static boolean removedRules;

	protected int m_ruleDirtySectionOrdering;
	protected String m_ruleType;
	protected IPAddress m_sourceAddress;
	protected IPMask m_sourceMask;
	protected IPAddress m_destinationAddress;
	protected IPMask m_destinationMask;
	protected String m_interfaceName;
	protected int  m_flags;
	protected String  m_policy;
	protected int  m_protocol;
	protected int  m_sourceLowPort;
	protected int  m_sourceHighPort;
	protected int  m_destinationLowPort;
	protected int  m_destinationHighPort;
	protected long m_packetCount;
	protected long m_byteCount;
	protected short m_TOSand;
	protected short m_TOSxor;
  protected boolean m_affectedToService;
  protected boolean m_isApplied;

/**
 * InputRule constructor.
 */
public InputOutputRule()
{
	super();
	m_sourceLowPort = 0;
	m_sourceHighPort = 65535;
	m_destinationLowPort = 0;
	m_destinationHighPort = 65535;

	m_ruleDirtySectionOrdering = Linux22.DIRTY_ORDERING_SECTION_2_BASE + Linux22.m_ruleDirtyNumber++;
}
public InputOutputRule(String raw)
{
	super();

	StringTokenizer st = new StringTokenizer(raw, " \t\r\n/");

	m_ruleDirtySectionOrdering = Linux22.DIRTY_ORDERING_SECTION_2_BASE + Linux22.m_ruleDirtyNumber++;
	m_ruleType = st.nextToken();
	m_sourceAddress = new IPAddress(Long.parseLong(st.nextToken(), 16));
	String sourceMaskArrowDestAddress = st.nextToken();
	StringTokenizer sad = new StringTokenizer(sourceMaskArrowDestAddress, "->");
	m_sourceMask = new IPMask(Long.parseLong(sad.nextToken(), 16));
	m_destinationAddress = new IPAddress(Long.parseLong(sad.nextToken(), 16));
	m_destinationMask = new IPMask(Long.parseLong(st.nextToken(), 16));
	m_interfaceName = st.nextToken();
	if (m_interfaceName.equals("-")) m_interfaceName = "all";

	m_flags = Integer.parseInt(st.nextToken(), 16);

	/*int kind = m_flags & Linux22.IP_FW_F_KIND;
	switch (kind)
	{
	case Linux22.IP_FW_F_TCP:
		m_protocol = Linux22.IP_FW_F_TCP;
		break;
	case Linux22.IP_FW_F_UDP:
		m_protocol = Linux22.IP_FW_F_UDP;
		break;
	case Linux22.IP_FW_F_ICMP:
		m_protocol = Linux22.IP_FW_F_ICMP;
		break;
	default:
		m_protocol = Linux22.IP_FW_F_ALL;
	}*/

	st.nextToken();
	m_protocol = Integer.parseInt(st.nextToken());
	for (int i=0; i < 4; i++) st.nextToken();

	String sourcePortsRange = st.nextToken();
	StringTokenizer sts = new StringTokenizer(sourcePortsRange, "-");
	m_sourceLowPort = Integer.parseInt(sts.nextToken());
	m_sourceHighPort = Integer.parseInt(sts.nextToken());
	
	String destinationPortsRange = st.nextToken();
	StringTokenizer std = new StringTokenizer(destinationPortsRange, "-");
	m_destinationLowPort = Integer.parseInt(std.nextToken());
	m_destinationHighPort = Integer.parseInt(std.nextToken());

	for (int i=0; i < 5; i++) st.nextToken();
	m_policy = st.nextToken();

	/*if ((m_flags & LinuxIPFWConstants.IP_FW_F_REDIR) != 0)
	m_policy = LinuxIPFWConstants.IP_FW_F_REDIR;
	else
	if ((m_flags & LinuxIPFWConstants.IP_FW_F_MASQ) != 0)
	m_policy = LinuxIPFWConstants.IP_FW_F_MASQ;
	else
	if ((m_flags & LinuxIPFWConstants.IP_FW_F_ACCEPT) != 0)
	m_policy = LinuxIPFWConstants.IP_FW_F_ACCEPT;
	else
	if ((m_flags & LinuxIPFWConstants.IP_FW_F_ICMPRPL) != 0)
	m_policy = LinuxIPFWConstants.IP_FW_F_ICMPRPL;
	else
	m_policy = 0;
	*/
}
public static boolean changesAreNotApplied()
{
  if (removedRules) return true;

  for (int i=0; i < miscRulesVector.size(); i++)
  {
	if(! ((InputOutputRule)miscRulesVector.elementAt(i)).m_isApplied) return true;
  }

  return false;
}
public int compareTo(Object o)
{
  return m_ruleDirtySectionOrdering - ((InputOutputRule)o).m_ruleDirtySectionOrdering;
}
public boolean destinationNetworkMatch(IPInterface ipi)
{
  if ((m_destinationAddress.getLong() & m_destinationMask.getLong()) == (ipi.getAddress().getLong() & ipi.getMask().getLong()))
	return true;

  return false;
}
public String getDestinationPortsString()
{
  if ((m_destinationLowPort == 0) && (m_destinationHighPort == 0)) m_destinationHighPort = 65535;

  if (m_destinationLowPort != m_destinationHighPort)
	return "" + m_destinationLowPort + ":" + m_destinationHighPort;

  return Linux22.getServiceName(m_destinationLowPort, getProtocolString());
}
public String getDestinationPortsStringForDisplay()
{
  if ((m_destinationLowPort == 0) && (m_destinationHighPort == 0)) return "all";
  if ((m_destinationLowPort == 0) && (m_destinationHighPort == 65535)) return "all";

  if (m_destinationLowPort != m_destinationHighPort)
	return "" + m_destinationLowPort + ":" + m_destinationHighPort;

  String res = Linux22.getServiceName(m_destinationLowPort, getProtocolString());
  try
  {
	int test = Integer.parseInt(res);
  }
  catch(NumberFormatException nfe)
  {
	res += " (" + m_destinationLowPort + ")";
  }

  return res;
}
public String getProtocolString()
{
  String res="";
	switch (m_protocol)
	{
	case Linux22.IPPROTO_TCP:
		res += ("tcp");
		break;
	case Linux22.IPPROTO_UDP:
		res += ("udp");
		break;
	case Linux22.IPPROTO_ICMP:
		res += ("icmp");
		break;
	case Linux22.IPPROTO_IP:
		res += ("all");
	}
  return res;
}
public String getRuleType()
{
	return m_ruleType;
}
public String getSourcePortsString()
{
  if ((m_sourceLowPort == 0) && (m_sourceHighPort == 0)) m_sourceHighPort = 65535;

  if (m_sourceLowPort != m_sourceHighPort)
	return "" + m_sourceLowPort + ":" + m_sourceHighPort;

  return Linux22.getServiceName(m_sourceLowPort, getProtocolString());
}
public String getSourcePortsStringForDisplay()
{
  if ((m_sourceLowPort == 0) && (m_sourceHighPort == 0)) return "all";
  if ((m_sourceLowPort == 0) && (m_sourceHighPort == 65535)) return "all";

  if (m_sourceLowPort != m_sourceHighPort)
	return "" + m_sourceLowPort + ":" + m_sourceHighPort;

  String res = Linux22.getServiceName(m_sourceLowPort, getProtocolString());
  try
  {
	int test = Integer.parseInt(res);
  }
  catch(NumberFormatException nfe)
  {
	res += " (" + m_sourceLowPort + ")";
  }

  return res;
}
public boolean isDestinationPort(int dport)
{
	if ((dport >= m_destinationLowPort) && (dport <= m_destinationHighPort)) return true;
	return false;
}
public boolean isSourcePort(int sport)
{
	if ((sport >= m_sourceLowPort) && (sport <= m_sourceHighPort)) return true;
	return false;
}
public static void makeChangesApplied()
{
	removedRules = false;
}
public boolean portsMatch(InputOutputRule ior)
{
	if (ior.m_destinationLowPort == m_sourceLowPort)
	if (ior.m_destinationHighPort == m_sourceHighPort)
	if (ior.m_sourceLowPort == m_destinationLowPort)
	if (ior.m_sourceHighPort == m_destinationHighPort)
	return true;

	return false;
}
public static void removeMiscRule(InputOutputRule ior)
{
	for (int i=0; i < InputOutputRule.miscRulesVector.size(); i++)
	{
		InputOutputRule ior2 = (InputOutputRule) InputOutputRule.miscRulesVector.elementAt(i);
		if (ior.toString().equals(ior2.toString()))
		{
			InputOutputRule.miscRulesVector.removeElementAt(i);
			InputOutputRule.removedRules = true;
			break;
		}
	}
	if (ior instanceof OutputRule)
	{
		for (int i=0; i < OutputRule.outputRulesVector.size(); i++)
		{
			InputOutputRule ior2 = (InputOutputRule) OutputRule.outputRulesVector.elementAt(i);
			if ((! ior2.m_affectedToService) && (ior.toString().equals(ior2.toString())))
			{
				OutputRule.outputRulesVector.removeElementAt(i);
				OutputRule.removedRules = true;
				break;
			}
		}
	}
	else
	{
		for (int i=0; i < InputRule.inputRulesVector.size(); i++)
		{
			InputOutputRule ior2 = (InputOutputRule) InputRule.inputRulesVector.elementAt(i);
			if ((! ior2.m_affectedToService) && (ior.toString().equals(ior2.toString())))
			{
				InputRule.inputRulesVector.removeElementAt(i);
				InputRule.removedRules = true;
				break;
			}
		}
	}
}
public static void reset()
{
  if (miscRulesVector.size() > 1) removedRules = true;
  miscRulesVector.removeAllElements();
}
public boolean sourceNetworkMatch(IPInterface ipi)
{
  if ((m_sourceAddress.getLong() & m_sourceMask.getLong()) == (ipi.getAddress().getLong() & ipi.getMask().getLong()))
	return true;

  return false;
}
public String toString()
{
	String res = m_ruleType;
	res += " " + m_policy;
	res += " SRC[" + m_sourceAddress.toString() + "/" + m_sourceMask.toString() + "]";
	res += " DST[" + m_destinationAddress.toString() + "/" + m_destinationMask.toString() + "]";
	res += " IF[" + m_interfaceName + "]";
	res += " PROT[" + getProtocolString() + "]";

	if ((m_flags & Linux22.IP_FW_F_REDIR) != 0)
	{
		res += (" [REDIR] ");
	}

  res += " SPORT[";
  if (m_sourceLowPort != m_sourceHighPort)
	res += m_sourceLowPort + ":" + m_sourceHighPort;
  else
	res += getSourcePortsStringForDisplay();
  res += "]";

  res += " DPORT[";
  if (m_destinationLowPort != m_destinationHighPort)
	res += m_destinationLowPort + ":" + m_destinationHighPort;
  else
	res += getDestinationPortsStringForDisplay();
  res += "]";

	return res;
}
}
