package fwadmin;

/*

  JFwadmin v 0.8

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;

public class OutputServicesPanel extends JPanel implements UpdateWindow, ActionListener
{
  String[] columnsNames = {"Output service / port(s)", "Interface", "Protocol", "Destination address", "Destination mask", "Policy", "Comment"};
  Object[][] data = {};
  Object[][] old_data = {};
  JTable m_tableView = null;
  JScrollPane m_scrollPane = null;




	private javax.swing.JPopupMenu m_jPopupMenu;
	private javax.swing.JMenuItem m_jMenuItemDeselect;
	private javax.swing.JMenuItem m_jMenuItemRemove;
	private javax.swing.JMenuItem m_jMenuItemAdd;
  
  public OutputServicesPanel()
  {
	this.setLayout(new BorderLayout());
	doDisplayTable();
  }      
public void actionPerformed(ActionEvent e)
{
	if (m_tableView.getSelectedRow() == -1) return;
	
	if (e.getSource().equals(m_jMenuItemDeselect))
	{
		m_tableView.clearSelection();
		return;
	}

	if (e.getSource().equals(m_jMenuItemRemove))
	{
		OutputService selectedService = (OutputService)data[m_tableView.getSelectedRow()][columnsNames.length];
		OutputService.removeOutputService(selectedService);
		JFwadmin.getInstance().refreshPanels();
		return;
	}

	if (e.getSource().equals(m_jMenuItemAdd))
	{
		JFwadmin.getInstance().new_output_serviceMenuItem_ActionEvents();
	}
}
  public void doDisplayTable()
  {
	// Create a model of the data.
	TableModel dataModel = new AbstractTableModel() {
	public int getColumnCount() { return columnsNames.length; }
	public int getRowCount() { return data.length;}
	public Object getValueAt(int row, int col) {return data[row][col];}
	public String getColumnName(int column) {return columnsNames[column];}
	public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
	public boolean isCellEditable(int row, int col) {return false;}
	public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
	};

	// Create the table
	m_tableView = new JTable(dataModel);
	m_tableView .setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	m_tableView.setRowHeight(InputServicesPanel.m_rowHeightInitialValue);
  	m_tableView.addMouseListener(new MouseAdapter() {
	public void mouseClicked(MouseEvent e)
	{
		if (m_tableView.getSelectedRow() == -1) return;

		if ((e.getModifiers() & e.BUTTON3_MASK) != 0)
		{
		  getJPopupMenu().show(m_tableView, e.getX(), e.getY());
		}
	}
	});

  	m_scrollPane = new JScrollPane(m_tableView);
	this.add("Center", m_scrollPane);
  }      
public JPopupMenu getJPopupMenu()
{
	if (m_jPopupMenu == null)
	{
		m_jPopupMenu = new JPopupMenu("Choose Action");
		m_jPopupMenu.setLabel("Choose Action");
		m_jMenuItemDeselect = new JMenuItem("Deselect");
		m_jMenuItemDeselect.addActionListener(this);
		m_jPopupMenu.add(m_jMenuItemDeselect);
		m_jMenuItemRemove = new JMenuItem("Remove");
		m_jMenuItemRemove.addActionListener(this);
		m_jPopupMenu.add(m_jMenuItemRemove);
		m_jMenuItemAdd = new JMenuItem("Add new service");
		m_jMenuItemAdd.addActionListener(this);
		m_jPopupMenu.add(m_jMenuItemAdd);
	}
	return m_jPopupMenu;
}
  public void updateWindow()
  {
	data = new Object[OutputService.outputServicesVector.size()][columnsNames.length + 1];
	for (int i=0; i < OutputService.outputServicesVector.size(); i++)
	{
	  OutputService os = (OutputService) OutputService.outputServicesVector.elementAt(i);
	  data[i][0] = os.m_or.getDestinationPortsStringForDisplay();
	  data[i][1] = os.m_or.m_interfaceName;
	  data[i][2] = os.m_or.getProtocolString();
	  data[i][3] = os.m_or.m_destinationAddress.toString();
	  data[i][4] = os.m_or.m_destinationMask.toString();
	  data[i][5] = os.m_or.m_policy;
	  data[i][6] = os.getComment();
	  data[i][columnsNames.length] = os;
	}

	// Avoid table blinking : verify data changed, and if not, don't update table.
	boolean differenceEncountered = false;
	if (data.length != old_data.length) differenceEncountered = true;
	if (! differenceEncountered)
	{
	  for(int i=0; i < data.length; i++)
	  {
		if (data[i].length != old_data[i].length) differenceEncountered = true;
		if (differenceEncountered) break;

		for (int j=0; j < data[i].length; j++)
		{
		  if (!(data[i][j].equals(old_data[i][j]))) differenceEncountered = true;
		  if (differenceEncountered) break;
		}
	  }
	}
	// No difference between new and old data, so nothing to do and return.
	if (! differenceEncountered) return;

	old_data = data;

	// Create a model of the data.
	TableModel dataModel = new AbstractTableModel() {
	public int getColumnCount() { return columnsNames.length; }
	public int getRowCount() { return data.length;}
	public Object getValueAt(int row, int col) {return data[row][col];}
	public String getColumnName(int column) {return columnsNames[column];}
	public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
	public boolean isCellEditable(int row, int col) {return false;}
	public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
	};


	m_tableView.setModel(dataModel);
	for(int i=0; i < columnsNames.length; i++)
	{
		if (! columnsNames[i].equals("Policy"))
			m_tableView.getColumn(columnsNames[i]).setCellRenderer(DefaultPolicy.getStringRenderer());
		else
			m_tableView.getColumn(columnsNames[i]).setCellRenderer(DefaultPolicy.getPolicyRenderer());
	}
	m_tableView.getColumn(columnsNames[0]).setPreferredWidth(100);
	m_tableView.getColumn("Policy").setPreferredWidth(30);
	m_tableView.getColumn("Protocol").setPreferredWidth(20);
	m_tableView.getColumn("Interface").setPreferredWidth(20);
	m_tableView.sizeColumnsToFit(-1);

  }          
}
